package org.apache.ojb.odmg.collections;

import org.odmg.DList;
import org.odmg.DBag;
import org.odmg.DSet;
import org.odmg.DArray;
import org.odmg.DMap;
import org.apache.ojb.broker.util.factory.ConfigurableFactory;
import org.apache.ojb.broker.PBKey;

/* Copyright 2003-2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
public class DCollectionFactory
{
    private static final DCollectionFactory singleton = new DCollectionFactory();

    private DCollectionFactory()
    {
    }

    public static DCollectionFactory getInstance()
    {
        return singleton;
    }

    public DList createDList()
    {
        return (DList) DListFactory.singleton.createCollectionOrMap();
    }

    public DList createDList(PBKey key)
    {
        return (DList) DListFactory.singleton.createCollectionOrMap(key);
    }

    public DBag createDBag()
    {
        return (DBag) DBagFactory.singleton.createCollectionOrMap();
    }

    public DBag createDBag(PBKey key)
    {
        return (DBag) DBagFactory.singleton.createCollectionOrMap(key);
    }

    public DSet createDSet()
    {
        return (DSet) DSetFactory.singleton.createCollectionOrMap();
    }

    public DSet createDSet(PBKey key)
    {
        return (DSet) DSetFactory.singleton.createCollectionOrMap(key);
    }

    public DArray createDArray()
    {
        return (DArray) DArrayFactory.singleton.createCollectionOrMap();
    }

    public DArray createDArray(PBKey key)
    {
        return (DArray) DArrayFactory.singleton.createCollectionOrMap(key);
    }

    public DMap createDMap()
    {
        return (DMap) DMapFactory.singleton.createCollectionOrMap();
    }

    public DMap createDMap(PBKey key)
    {
        return (DMap) DMapFactory.singleton.createCollectionOrMap(key);
    }

    //*****************************************************
    // inner classes
    //*****************************************************

    abstract static class BaseFactory extends ConfigurableFactory
    {
        Object createCollectionOrMap()
        {
            return this.createNewInstance();
        }

        Object createCollectionOrMap(PBKey key)
        {
            return createNewInstance(PBKey.class, key);
        }
    }

    static final class DListFactory extends BaseFactory
    {
        static final BaseFactory singleton = new DListFactory();
        protected String getConfigurationKey()
        {
            return "DListClass";
        }
    }

    static final class DArrayFactory extends BaseFactory
    {
        static final BaseFactory singleton = new DArrayFactory();
        protected String getConfigurationKey()
        {
            return "DArrayClass";
        }
    }

    static final class DBagFactory extends BaseFactory
    {
        static final BaseFactory singleton = new DBagFactory();
        protected String getConfigurationKey()
        {
            return "DBagClass";
        }
    }

    static final class DSetFactory extends BaseFactory
    {
        static final BaseFactory singleton = new DSetFactory();
        protected String getConfigurationKey()
        {
            return "DSetClass";
        }
    }

    static final class DMapFactory extends BaseFactory
    {
        static final BaseFactory singleton = new DMapFactory();
        protected String getConfigurationKey()
        {
            return "DMapClass";
        }
    }
}
